'******************************************************************************************
'******************************************************************************************
'PADS PowerPCB VB macro  manhattan103.bas   Updated: 18-01-07  Ron Olinyk
'Copyright (c) 2007 Mindlink Technologies, All Rights Reserved.  www.mindlinktech.com
'
'This script should work in all versions of PowerPCB from 2.1 onwards.
'Please report any errors or bugs to support@mindlinktech.com
'
'Terms of Use: 
' This sofware is distributed free of charge, compliments of Mindlink Technologies. 
' This software is provided "as is" with no guarantee of accuracy and without warranty of 
' any kind, express or implied. 
' Copies of this script may not be uploaded to forums or placed on web pages, CDs, ect. 
' without the author's permission. Please provide a link only to the script's home page.
'Description: 
' Generates a report of the manhattan lengths for nets in the design. Report 
' format is similar to that generated by Specctra.  Routed and unrouted lengths are also 
' displayed along with the manhattan to design length ratio and a length difference between
' these two values. The report is saved to a file in the design directory and loaded to Notepad 
' for viewing.  All lengths are reported in current design units.
'Usage: 
' Select nets (if no nets selected, all will be reported on). To display manhattan length
' on pin pairs in net as well, check on 'Report pin pair lengths' checkbox. To include nets 
' assigned to planes, check on 'Include plane nets' checkbox. Click OK. Report displayed in Notepad.
' Additional reports can be run. Save file to another name as it will be overwritten. Cancel button 
' quits macro script.
'History: 
' 12-12-03 manhattan101 -initial release
' 23-02-05 manhattan102 -bug fix to report on zero length nets due to coincident SMT pads
' 18-01-07 manhattan103 -minor program updates (non-bug related)

Option Explicit
Private skip_pwr As Boolean
Private ntmr As Double
Private incldPP As Boolean
Private proces As Boolean
Private Const pver = 1.03
Private d_unt As Integer
Private dunam As String

Sub main
   skip_pwr = False
   proces = True
	Begin Dialog UserDialog 350,203,"Manhattan Net Length Reporting " & pver,.ManHat_Rep ' %GRID:10,7,1,1
		GroupBox 30,28,290,84,"",.GroupBox1
		CheckBox 60,56,20,14,"",.cbxPPair
		CheckBox 60,77,20,14,"",.cbxPlnNets
		Text 100,56,190,14,"Report pin pair lengths",.Text1
		Text 100,77,130,14,"Include plane nets",.Text2
		Text 30,133,290,28,"",.Errormess
		Text 30,14,300,14,"",.TxtNote
		OKButton 60,168,90,21
		CancelButton 200,168,90,21
	End Dialog
	Dim dlg As UserDialog
	Dialog dlg
End Sub

Sub Manhatn_Rep
Dim pp As Object
Dim all_nets As Object
Dim xnet As Object
Dim sp As String
Dim sp6 As String
Dim fn As String
Dim fhdl As Integer
Dim nfmt As String
Dim manh As Double
Dim t_len As Double
Dim r_len As Double
Dim n_len As String
Dim npins As Long
Dim nvias As Long
Dim nconn As Long
Dim xstr As String
Dim xdif As Double
Dim ydif As Double
Dim pp_manh As Double
Dim titl As String
Dim nrnd As Integer
Dim nnam As Integer
Dim sp2 As String
Dim App As Variant
Dim zlen As Boolean

On Error GoTo AppErr
proces = True
sp = Space(3)
sp6 = Space(6)
fn = ActiveDocument.path & "\Net_Manhattan.rep"
fhdl = FreeFile
Open fn For Output As #fhdl
nfmt = Choose(d_unt, "0", "#####0.0", "#0.0000", "##0.000")		  
Print #fhdl, "Manhattan Net Length Report" & sp & ActiveDocument.Name & sp & Date & sp & "Units: " & dunam
Print #fhdl, String(60, 45) & vbCrLf
titl = "Net              Pins  Vias  Conns  Manhattan  Routed  Unrouted  Ratio(R+U/M) Len Diff(R+U-M)"
Print #fhdl, titl
Set all_nets = ActiveDocument.GetObjects(ppcbObjectTypeNet,,True)

If all_nets.Count = 0 Then
   Set all_nets = ActiveDocument.GetObjects(ppcbObjectTypeNet)
End If

For Each xnet In all_nets
  If xnet.Power And skip_pwr Then proces = False
  If proces Then
   manh = 0
   xstr = ""
   npins = xnet.Pins.Count
   nvias = xnet.Vias.Count
   nconn = xnet.Connections.Count
   t_len = xnet.Length
   r_len = xnet.Length(True)
   n_len = Format(r_len, nfmt) & sp & Format((t_len - r_len), nfmt)
   For Each pp In xnet.Connections
      If incldPP Then xstr = xstr & vbCrLf
      xdif = Abs(pp.Pins(1).PositionX - pp.Pins(2).PositionX)
      ydif = Abs(pp.Pins(1).PositionY - pp.Pins(2).PositionY)
      pp_manh = xdif + ydif
      manh = manh + pp_manh
      If incldPP Then xstr = xstr & sp & pp & sp & Format(pp_manh, nfmt)
   Next pp
   If manh > 0 Then
      n_len = n_len & sp6 & Format((t_len/manh), nfmt) & Space(9) & Format((t_len - manh), nfmt)
   Else
      n_len = n_len & sp6 & "0 *" & Space(9) & "0 *"
      zlen = True      
   End If   
   nnam = Len(xnet)
   If nnam < 20 Then
      sp2 = Space(20 - nnam)
   Else
      sp2 = sp
   End If   
   Print #fhdl, xnet & sp2 & npins & sp & nvias & sp6 & nconn & sp6 & Format(manh, nfmt) & sp & n_len & xstr
  End If 
  proces = True
Next xnet
If zlen Then
   Print #fhdl,
   Print #fhdl, "* Net is zero length -possible coincident pads."
End If  
Close #fhdl
On Error GoTo AppErr
App = Shell ("Notepad " & fn, 1)
On Error Resume Next
AppActivate App
Exit Sub
AppErr:
MsgBox "Process Error" & vbCr _
& "Exiting...", vbExclamation, "Error msg"
	
End Sub

Sub Chk_Design
	   If ActiveDocument.Nets.Count = 0 Then
          DlgText "Errormess","Empty design -no nets present."
          proces = False
       Else
          proces = True
       End If
       d_unt = ActiveDocument.unit
	   dunam = Choose(d_unt, "0", "mil", "inch", "mm")
End Sub

Private Function ManHat_Rep(DlgItem$, Action%, SuppValue&) As Boolean
Const mlmsg = "Courtesy of: Mindlink Technologies" & vbCr & Space(20) & "www.mindlinktech.com"
	Select Case Action%
	Case 1 
	    DlgText "Errormess", mlmsg
	    DlgText "TxtNote", "Note: Select nets (reports all if none selected)"
	    Chk_Design     
	Case 2 
	     Select Case dlgitem
	       Case "Cancel"
	         End
	       Case "OK"
	         DlgText "Errormess", "Processing nets...standby..."
	         Chk_Design
	        If proces Then
	         skip_pwr = IIf(DlgValue("cbxPlnNets") = 0, True, False) 
	         incldPP = IIf(DlgValue("cbxPPair") = 1, True, False) 
	         Manhatn_Rep
	         DlgText "Errormess", "Report completed"
	         Wait 3
	         DlgText "Errormess", mlmsg
	        End If 
	     End Select  
		 ManHat_Rep = True 
	End Select
End Function

